@@ -179,7 +179,7 @@ class Database(DatabaseInterface):
         res = Database._list_cache
         if res and abs(Database._list_cache_timestamp - now) < timeout:
             return res
-        db_user = CONFIG['db_user']
+        db_user = CONFIG['db_user'] or os.environ.get('PGUSER')
         if not db_user and os.name == 'posix':
             db_user = pwd.getpwuid(os.getuid())[0]
         if not db_user:
@@ -353,7 +353,7 @@ class Cursor(CursorInterface):
         return self.cursor.fetchone()[0]
 
     def lock(self, table):
-        self.cursor.execute('LOCK "%s" IN EXCLUSIVE MODE' % table)
+        self.cursor.execute('LOCK "%s" IN EXCLUSIVE MODE NOWAIT' % table)
 
     def has_constraint(self):
         return True
