@@ -8,9 +8,13 @@ import logging
 import traceback
 import sys
 import re
-from trytond.version import VERSION
-from trytond.tools import safe_eval
-from trytond.transaction import Transaction
+from sql import Table
+from itertools import izip
+from collections import defaultdict
+
+from .version import VERSION
+from .tools import safe_eval
+from .transaction import Transaction
 
 CDATA_START = re.compile('^\s*\<\!\[cdata\[', re.IGNORECASE)
 CDATA_END = re.compile('\]\]\>\s*$', re.IGNORECASE)
@@ -61,26 +65,37 @@ class MenuitemTagHandler:
             action_id = self.mh.get_id(values['action'])
 
             # TODO maybe use a prefetch for this:
-            cursor.execute(cursor.limit_clause(
-                    "SELECT a.name, a.type, v.type, v.field_childs, icon.name "
-                    "FROM ir_action a "
-                        "LEFT JOIN ir_action_report report "
-                            "ON (a.id = report.action) "
-                        "LEFT JOIN ir_action_act_window act "
-                            "ON (a.id = act.action) "
-                        "LEFT JOIN ir_action_wizard wizard "
-                            "ON (a.id = wizard.action) "
-                        "LEFT JOIN ir_action_url url ON (a.id = url.action) "
-                        "LEFT JOIN ir_action_act_window_view wv ON "
-                            "(act.id = wv.act_window) "
-                        "LEFT JOIN ir_ui_view v ON (v.id = wv.view) "
-                        "LEFT JOIN ir_ui_icon icon ON (a.icon = icon.id) "
-                    "WHERE report.id = %s "
-                        "OR act.id = %s "
-                        "OR wizard.id = %s "
-                        "OR url.id = %s "
-                    "ORDER by wv.sequence", 1),
-                (action_id, action_id, action_id, action_id))
+            action = Table('ir_action')
+            report = Table('ir_action_report')
+            act_window = Table('ir_action_act_window')
+            wizard = Table('ir_action_wizard')
+            url = Table('ir_action_url')
+            act_window_view = Table('ir_action_act_window_view')
+            view = Table('ir_ui_view')
+            icon = Table('ir_ui_icon')
+            cursor.execute(*action.join(
+                    report, 'LEFT',
+                    condition=action.id == report.action
+                    ).join(act_window, 'LEFT',
+                    condition=action.id == act_window.action
+                    ).join(wizard, 'LEFT',
+                    condition=action.id == wizard.action
+                    ).join(url, 'LEFT',
+                    condition=action.id == url.action
+                    ).join(act_window_view, 'LEFT',
+                    condition=act_window.id == act_window_view.act_window
+                    ).join(view, 'LEFT',
+                    condition=view.id == act_window_view.view
+                    ).join(icon, 'LEFT',
+                    condition=action.icon == icon.id).select(
+                    action.name, action.type,
+                    view.type, view.field_childs,
+                    icon.name,
+                    where=(report.id == action_id)
+                    | (act_window.id == action_id)
+                    | (wizard.id == action_id)
+                    | (url.id == action_id),
+                    order_by=act_window_view.sequence, limit=1))
             action_name, action_type, view_type, field_childs, icon_name = \
                 cursor.fetchone()
 
@@ -122,6 +137,9 @@ class MenuitemTagHandler:
             else:
                 values['name'] = action_name
 
+        if values.get('sequence'):
+            values['sequence'] = int(values['sequence'])
+
         self.values = values
 
     def characters(self, data):
@@ -374,7 +392,7 @@ class Fs2bdAccessor:
 
 class TrytondXmlHandler(sax.handler.ContentHandler):
 
-    def __init__(self, pool, module,):
+    def __init__(self, pool, module, module_state):
         "Register known taghandlers, and managed tags."
         sax.handler.ContentHandler.__init__(self)
 
@@ -384,7 +402,16 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
         self.fs2db = Fs2bdAccessor(self.ModelData, pool)
         self.to_delete = self.populate_to_delete()
         self.noupdate = None
+        self.module_state = module_state
+        self.grouped = None
+        self.grouped_creations = defaultdict(dict)
+        self.grouped_write = defaultdict(list)
+        self.grouped_model_data = []
         self.skip_data = False
+        Module = pool.get('ir.module.module')
+        self.installed_modules = [m.name for m in Module.search([
+                    ('state', 'in', ['installed', 'to upgrade']),
+                    ])]
 
         # Tag handlders are used to delegate the processing
         self.taghandlerlist = {
@@ -430,11 +457,17 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
 
             elif name == "data":
                 self.noupdate = bool(int(attributes.get("noupdate", '0')))
+                self.grouped = bool(int(attributes.get('grouped', 0)))
                 if self.pool.test and \
                         bool(int(attributes.get("skiptest", '0'))):
                     self.skip_data = True
                 else:
                     self.skip_data = False
+                depends = attributes.get('depends', '').split(',')
+                depends = [m.strip() for m in depends if m]
+                if depends:
+                    if not all((m in self.installed_modules for m in depends)):
+                        self.skip_data = True
 
             elif name == "tryton":
                 pass
@@ -452,10 +485,24 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
 
     def endElement(self, name):
 
+        if name == 'data' and self.grouped:
+            for model, values in self.grouped_creations.iteritems():
+                self.create_records(model, values.values(), values.keys())
+            self.grouped_creations.clear()
+            for key, actions in self.grouped_write.iteritems():
+                module, model = key
+                self.write_records(module, model, *actions)
+            self.grouped_write.clear()
+        if self.grouped_model_data:
+            self.ModelData.write(*self.grouped_model_data)
+            del self.grouped_model_data[:]
+
         # Closing tag found, if we are in a delegation the handler
         # know what to do:
         if self.taghandler and not self.skip_data:
             self.taghandler = self.taghandler.endElement(name)
+        if self.taghandler == self.taghandlerlist.get(name):
+            self.taghandler = None
 
     def current_state(self):
         if self.taghandler:
@@ -545,7 +592,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
             if module == self.module and fs_id in self.to_delete:
                 self.to_delete.remove(fs_id)
 
-            if self.noupdate:
+            if self.noupdate and self.module_state != 'to install':
                 return
 
             # this record is already in the db:
@@ -638,67 +685,101 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                 # and no user changed the value in the db:
                 to_update[key] = values[key]
 
-            # if there is values to update:
-            if to_update:
-                # write the values in the db:
-                with Transaction().set_context(
-                        module=module, language='en_US'):
-                    Model.write([record], to_update)
-                self.fs2db.reset_browsercord(
-                    module, Model.__name__, [record.id])
-
-            if to_update:
-                # re-read it: this ensure that we store the real value
-                # in the model_data table:
-                record = self.fs2db.get_browserecord(
-                    module, Model.__name__, record.id)
-                if not record:
-                    record = Model(record.id)
-                for key in to_update:
-                    values[key] = self._clean_value(key, record)
-
-            if module != self.module:
-                temp_values = old_values.copy()
-                temp_values.update(values)
-                values = temp_values
-
-            if values != old_values:
-                self.ModelData.write([self.ModelData(mdata_id)], {
-                    'fs_id': fs_id,
-                    'model': model,
-                    'module': module,
-                    'db_id': record.id,
-                    'values': str(values),
-                    'date_update': datetime.datetime.now(),
-                    })
-            # reset_browsercord to keep cache memory low
-            self.fs2db.reset_browsercord(module, Model.__name__, [record.id])
-
+            if self.grouped:
+                self.grouped_write[(module, model)].extend(
+                    (record, to_update, old_values, fs_id, mdata_id))
+            else:
+                self.write_records(module, model, record, to_update,
+                    old_values, fs_id, mdata_id)
         else:
-            # this record is new, create it in the db:
-            with Transaction().set_context(module=module, language='en_US'):
-                record, = Model.create([values])
+            if self.grouped:
+                self.grouped_creations[model][fs_id] = values
+            else:
+                self.create_records(model, [values], [fs_id])
 
-            # re-read it: this ensure that we store the real value
-            # in the model_data table:
-            record = Model(record.id)
+    def create_records(self, model, vlist, fs_ids):
+        Model = self.pool.get(model)
+
+        with Transaction().set_context(module=self.module, language='en_US'):
+            records = Model.create(vlist)
+
+        mdata_values = []
+        for record, values, fs_id in izip(records, vlist, fs_ids):
             for key in values:
                 values[key] = self._clean_value(key, record)
 
-            mdata, = self.ModelData.create([{
-                        'fs_id': fs_id,
-                        'model': model,
-                        'module': module,
-                        'db_id': record.id,
-                        'values': str(values),
-                        'noupdate': self.noupdate,
-                        }])
+            mdata_values.append({
+                    'fs_id': fs_id,
+                    'model': model,
+                    'module': self.module,
+                    'db_id': record.id,
+                    'values': str(values),
+                    'noupdate': self.noupdate,
+                    })
 
-            # update fs2db:
-            self.fs2db.set(module, fs_id, {
-                    "db_id": record.id, "model": model,
-                    "id": mdata.id, "values": str(values)})
-            self.fs2db.reset_browsercord(module, model, [record.id])
+        models_data = self.ModelData.create(mdata_values)
+
+        for record, values, fs_id, mdata in izip(
+                records, vlist, fs_ids, models_data):
+            self.fs2db.set(self.module, fs_id, {
+                    'db_id': record.id,
+                    'model': model,
+                    'id': mdata.id,
+                    'values': str(values),
+                    })
+        self.fs2db.reset_browsercord(self.module, model,
+            [r.id for r in records])
+
+    def write_records(self, module, model,
+            record, values, old_values, fs_id, mdata_id, *args):
+        args = (record, values, old_values, fs_id, mdata_id) + args
+        Model = self.pool.get(model)
+
+        actions = iter(args)
+        to_update = []
+        for record, values, _, _, _ in zip(*((actions,) * 5)):
+            if values:
+                to_update += [[record], values]
+        # if there is values to update:
+        if to_update:
+            # write the values in the db:
+            with Transaction().set_context(
+                    module=module, language='en_US'):
+                Model.write(*to_update)
+            self.fs2db.reset_browsercord(
+                module, Model.__name__, sum(to_update[::2], []))
+
+        actions = iter(to_update)
+        for records, values in zip(actions, actions):
+            record, = records
+            # re-read it: this ensure that we store the real value
+            # in the model_data table:
+            record = self.fs2db.get_browserecord(
+                module, Model.__name__, record.id)
+            if not record:
+                record = Model(record.id)
+            for key in values:
+                values[key] = self._clean_value(key, record)
+
+        actions = iter(args)
+        for record, values, old_values, fs_id, mdata_id in zip(
+                *((actions,) * 5)):
+            temp_values = old_values.copy()
+            temp_values.update(values)
+            values = temp_values
+
+            if values != old_values:
+                self.grouped_model_data.extend(([self.ModelData(mdata_id)], {
+                            'fs_id': fs_id,
+                            'model': model,
+                            'module': module,
+                            'db_id': record.id,
+                            'values': str(values),
+                            'date_update': datetime.datetime.now(),
+                            }))
+
+        # reset_browsercord to keep cache memory low
+        self.fs2db.reset_browsercord(module, Model.__name__, args[::5])
 
 
 def post_import(pool, module, to_delete):
