@@ -6,9 +6,12 @@ from operator import itemgetter
 from collections import defaultdict
 from functools import partial
 
+from sql import Table
+from sql.aggregate import Count
+
 from ..model import ModelView, ModelStorage, ModelSQL, fields
 from ..tools import file_open, safe_eval
-from ..backend import TableHandler
+from .. import backend
 from ..pyson import PYSONEncoder, CONTEXT, PYSON
 from ..transaction import Transaction
 from ..pool import Pool
@@ -53,9 +56,9 @@ class Action(ModelSQL, ModelView):
         return True
 
     @classmethod
-    def write(cls, actions, values):
+    def write(cls, actions, values, *args):
         pool = Pool()
-        super(Action, cls).write(actions, values)
+        super(Action, cls).write(actions, values, *args)
         pool.get('ir.action.keyword')._get_keyword_cache.clear()
 
     @classmethod
@@ -124,6 +127,7 @@ class ActionKeyword(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         super(ActionKeyword, cls).__register__(module_name)
 
         table = TableHandler(Transaction().cursor, cls, module_name)
@@ -190,9 +194,12 @@ class ActionKeyword(ModelSQL, ModelView):
         return super(ActionKeyword, cls).create(new_vlist)
 
     @classmethod
-    def write(cls, keywords, vals):
-        vals = cls._convert_vals(vals)
-        super(ActionKeyword, cls).write(keywords, vals)
+    def write(cls, keywords, values, *args):
+        actions = iter((keywords, values) + args)
+        args = []
+        for keywords, values in zip(actions, actions):
+            args.extend((keywords, cls._convert_vals(values)))
+        super(ActionKeyword, cls).write(*args)
         ModelView._fields_view_get_cache.clear()
         ModelView._view_toolbar_get_cache.clear()
         cls._get_keyword_cache.clear()
@@ -219,6 +226,7 @@ class ActionKeyword(ModelSQL, ModelView):
                     ('model', '=', model + ',' + str(model_id)),
                     ],
                 ]
+        clause = [clause, ('action.active', '=', True)]
         action_keywords = cls.search(clause, order=[])
         types = defaultdict(list)
         for action_keyword in action_keywords:
@@ -291,6 +299,7 @@ class ActionMixin(ModelSQL):
     def create(cls, vlist):
         pool = Pool()
         Action = pool.get('ir.action')
+        ir_action = cls.__table__()
         new_records = []
         for values in vlist:
             later = {}
@@ -307,11 +316,15 @@ class ActionMixin(ModelSQL):
             cursor = Transaction().cursor
             if cursor.nextid(cls._table):
                 cursor.setnextid(cls._table, cursor.currid(Action._table))
-            action, = Action.create([action_values])
-            values['action'] = action.id
+            if 'action' not in values:
+                action, = Action.create([action_values])
+                values['action'] = action.id
+            else:
+                action = Action(values['action'])
             record, = super(ActionMixin, cls).create([values])
-            cursor.execute('UPDATE "' + cls._table + '" SET id = %s '
-                'WHERE id = %s', (action.id, record.id))
+            cursor.execute(*ir_action.update(
+                    [ir_action.id], [action.id],
+                    where=ir_action.id == record.id))
             cursor.update_auto_increment(cls._table, action.id)
             record = cls(action.id)
             new_records.append(record)
@@ -319,10 +332,10 @@ class ActionMixin(ModelSQL):
         return new_records
 
     @classmethod
-    def write(cls, records, values):
+    def write(cls, records, values, *args):
         pool = Pool()
         ActionKeyword = pool.get('ir.action.keyword')
-        super(ActionMixin, cls).write(records, values)
+        super(ActionMixin, cls).write(records, values, *args)
         ActionKeyword._get_keyword_cache.clear()
 
     @classmethod
@@ -355,8 +368,12 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
     report_name = fields.Char('Internal Name', required=True)
     report = fields.Char('Path')
     report_content_custom = fields.Binary('Content')
-    report_content = fields.Function(fields.Binary('Content'),
-            'get_report_content', setter='set_report_content')
+    report_content = fields.Function(fields.Binary('Content',
+            filename='report_content_name'),
+        'get_report_content', setter='set_report_content')
+    report_content_name = fields.Function(fields.Char('Content Name',
+            on_change_with=['name', 'template_extension']),
+        'on_change_with_report_content_name')
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
     style = fields.Property(fields.Char('Style',
@@ -454,21 +471,24 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         super(ActionReport, cls).__register__(module_name)
 
         cursor = Transaction().cursor
         table = TableHandler(cursor, cls, module_name)
+        action_report = cls.__table__()
 
         # Migration from 1.0 report_name_uniq has been removed
         table.drop_constraint('report_name_uniq')
 
         # Migration from 1.0 output_format (m2o) is now extension (selection)
         if table.column_exist('output_format'):
-            cursor.execute(
-                'SELECT report.id FROM "' + cls._table + '" report '
-                'JOIN ir_action_report_outputformat of '
-                    'ON (report.output_format = of.id) '
-                'WHERE of.format = \'pdf\'')
+            outputformat = Table('ir_action_report_outputformat')
+            cursor.execute(*action_report.join(
+                    outputformat,
+                    condition=action_report.output_format == outputformat.id)
+                .select(action_report.id,
+                    where=outputformat.format == 'pdf'))
 
             ids = [x[0] for x in cursor.fetchall()]
             with Transaction().set_user(0):
@@ -478,34 +498,35 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
 
             table.drop_column("output_format")
             TableHandler.dropTable(cursor, 'ir.action.report.outputformat',
-                      'ir_action_report_outputformat')
+                'ir_action_report_outputformat')
 
         # Migrate from 2.0 remove required on extension
         table.not_null_action('extension', action='remove')
-        cursor.execute('UPDATE "' + cls._table + '" '
-            'SET extension = %s '
-            'WHERE extension = %s', ('', 'odt'))
+        cursor.execute(*action_report.update(
+                [action_report.extension],
+                [''],
+                where=action_report.extension == 'odt'))
 
         # Migration from 2.0 report_content_data renamed into
         # report_content_custom to remove base64 encoding
         if (table.column_exist('report_content_data')
                 and table.column_exist('report_content_custom')):
             limit = cursor.IN_MAX
-            cursor.execute('SELECT COUNT(id) '
-                'FROM "' + cls._table + '"')
+            cursor.execute(*action_report.select(
+                    Count(action_report.id)))
             report_count, = cursor.fetchone()
             for offset in range(0, report_count, limit):
-                cursor.execute(cursor.limit_clause(
-                    'SELECT id, report_content_data '
-                    'FROM "' + cls._table + '"'
-                    'ORDER BY id',
-                    limit, offset))
+                cursor.execute(*action_report.select(
+                        action_report.id, action_report.report_content_data,
+                        order_by=action_report.id,
+                        limit=limit, offset=offset))
                 for report_id, report in cursor.fetchall():
                     if report:
                         report = buffer(base64.decodestring(str(report)))
-                        cursor.execute('UPDATE "' + cls._table + '" '
-                            'SET report_content_custom = %s '
-                            'WHERE id = %s', (report, report_id))
+                        cursor.execute(*action_report.update(
+                                [action_report.report_content_custom],
+                                [report],
+                                where=action_report.id == report_id))
             table.drop_column('report_content_data')
 
     @staticmethod
@@ -581,6 +602,11 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
     def set_report_content(cls, records, name, value):
         cls.write(records, {'%s_custom' % name: value})
 
+    def on_change_with_report_content_name(self, name=None):
+        if not self.name:
+            return
+        return ''.join([self.name, os.extsep, self.template_extension])
+
     @classmethod
     def get_style_content(cls, reports, name):
         contents = {}
@@ -631,13 +657,18 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
         return new_reports
 
     @classmethod
-    def write(cls, reports, vals):
+    def write(cls, reports, values, *args):
         context = Transaction().context
         if 'module' in context:
-            vals = vals.copy()
-            vals['module'] = context['module']
-
-        super(ActionReport, cls).write(reports, vals)
+            actions = iter((reports, values) + args)
+            args = []
+            for reports, values in zip(actions, actions):
+                values = values.copy()
+                values['module'] = context['module']
+                args.extend((reports, values))
+            reports, values = args[:2]
+            args = args[2:]
+        super(ActionReport, cls).write(reports, values, *args)
 
 
 class ActionActWindow(ActionMixin, ModelSQL, ModelView):
@@ -645,6 +676,7 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     __name__ = 'ir.action.act_window'
     domain = fields.Char('Domain Value')
     context = fields.Char('Context Value')
+    order = fields.Char('Order Value')
     res_model = fields.Char('Model')
     act_window_views = fields.One2Many('ir.action.act_window.view',
             'act_window', 'Views')
@@ -654,8 +686,6 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     domains = fields.Function(fields.Binary('Domains'), 'get_domains')
     limit = fields.Integer('Limit', required=True,
             help='Default limit for the list view')
-    auto_refresh = fields.Integer('Auto-Refresh', required=True,
-            help='Add an auto-refresh on the view')
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
     window_name = fields.Boolean('Window Name',
@@ -665,6 +695,7 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     pyson_domain = fields.Function(fields.Char('PySON Domain'), 'get_pyson')
     pyson_context = fields.Function(fields.Char('PySON Context'),
             'get_pyson')
+    pyson_order = fields.Function(fields.Char('PySON Order'), 'get_pyson')
     pyson_search_value = fields.Function(fields.Char(
         'PySON Search Criteria'), 'get_pyson')
 
@@ -686,13 +717,19 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     @classmethod
     def __register__(cls, module_name):
         cursor = Transaction().cursor
+        TableHandler = backend.get('TableHandler')
+        act_window = cls.__table__()
         super(ActionActWindow, cls).__register__(module_name)
 
+        table = TableHandler(cursor, cls, module_name)
+
         # Migration from 2.0: new search_value format
-        cursor.execute('UPDATE "%s" '
-            'SET search_value = %%s '
-            'WHERE search_value = %%s' % cls._table,
-            ('[]', '{}'))
+        cursor.execute(*act_window.update(
+                [act_window.search_value], ['[]'],
+                where=act_window.search_value == '{}'))
+
+        # Migration from 3.0: auto_refresh removed
+        table.drop_column('auto_refresh')
 
     @staticmethod
     def default_type():
@@ -706,10 +743,6 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     def default_limit():
         return 0
 
-    @staticmethod
-    def default_auto_refresh():
-        return 0
-
     @staticmethod
     def default_window_name():
         return True
@@ -877,6 +910,7 @@ class ActionActWindowView(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         super(ActionActWindowView, cls).__register__(module_name)
         table = TableHandler(Transaction().cursor, cls, module_name)
 
@@ -891,9 +925,9 @@ class ActionActWindowView(ModelSQL, ModelView):
         return windows
 
     @classmethod
-    def write(cls, windows, values):
+    def write(cls, windows, values, *args):
         pool = Pool()
-        super(ActionActWindowView, cls).write(windows, values)
+        super(ActionActWindowView, cls).write(windows, values, *args)
         pool.get('ir.action.keyword')._get_keyword_cache.clear()
 
     @classmethod
